@@ -71,6 +71,8 @@ static void        popup_open_link                (GtkMenuItem        *item,
                                                    YelpView           *view);
 static void        popup_open_link_new            (GtkMenuItem        *item,
                                                    YelpView           *view);
+static void        popup_copy_link                (GtkMenuItem        *item,
+                                                   YelpView           *view);
 static void        popup_save_image               (GtkMenuItem        *item,
                                                    YelpView           *view);
 static void        popup_send_image               (GtkMenuItem        *item,
@@ -833,8 +835,13 @@ view_install_uri (YelpView    *view,
         g_variant_builder_unref (strv);
     }
     else {
+        gchar **pkgs;
+        gint i;
         strv = g_variant_builder_new (G_VARIANT_TYPE ("as"));
-        g_variant_builder_add (strv, "s", pkg);
+        pkgs = g_strsplit (pkg, ",", 0);
+        for (i = 0; pkgs[i]; i++)
+            g_variant_builder_add (strv, "s", pkgs[i]);
+        g_strfreev (pkgs);
         g_dbus_connection_call (connection,
                                 "org.freedesktop.PackageKit",
                                 "/org/freedesktop/PackageKit",
@@ -938,6 +945,16 @@ popup_open_link_new (GtkMenuItem *item,
     g_object_unref (uri);
 }
 
+static void
+popup_copy_link (GtkMenuItem *item,
+                 YelpView    *view)
+{
+    YelpViewPrivate *priv = GET_PRIV (view);
+    gtk_clipboard_set_text (gtk_widget_get_clipboard (GTK_WIDGET (view), GDK_SELECTION_CLIPBOARD),
+                            priv->popup_link_uri,
+                            -1);
+}
+
 typedef struct _YelpSaveData YelpSaveData;
 struct _YelpSaveData {
     GFile     *orig;
@@ -1214,7 +1231,7 @@ view_populate_popup (YelpView *view,
 
             child = (WebKitDOMNode *)
                 webkit_dom_element_query_selector (WEBKIT_DOM_ELEMENT (link_node),
-                                                   "div.linkdiv div.title", NULL);
+                                                   "span.title", NULL);
             if (child != NULL)
                 priv->popup_link_text = webkit_dom_node_get_text_content (child);
 
@@ -1246,18 +1263,25 @@ view_populate_popup (YelpView *view,
         }
 
         if (g_str_has_prefix (priv->popup_link_uri, "mailto:")) {
+            gchar *label = g_strdup_printf (_("Send email to %s"),
+                                            priv->popup_link_uri + 7);
             /* Not using a mnemonic because underscores are common in email
              * addresses, and we'd have to escape them. There doesn't seem
              * to be a quick GTK+ function for this. In practice, there will
              * probably only be one menu item for mailto link popups anyway,
              * so the mnemonic's not that big of a deal.
              */
-            gchar *label = g_strdup_printf (_("Send email to %s"),
-                                            priv->popup_link_uri + 7);
             item = gtk_menu_item_new_with_label (label);
             g_signal_connect (item, "activate",
                               G_CALLBACK (popup_open_link), view);
             gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
+            g_free (label);
+        }
+        else if (g_str_has_prefix (priv->popup_link_uri, "install:")) {
+            item = gtk_menu_item_new_with_mnemonic (_("_Install Packages"));
+            g_signal_connect (item, "activate",
+                              G_CALLBACK (popup_open_link), view);
+            gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
         }
         else {
             GSList *cur;
@@ -1267,10 +1291,19 @@ view_populate_popup (YelpView *view,
                               G_CALLBACK (popup_open_link), view);
             gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
 
-            item = gtk_menu_item_new_with_mnemonic (_("Open Link in New _Window"));
-            g_signal_connect (item, "activate",
-                              G_CALLBACK (popup_open_link_new), view);
-            gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
+            if (g_str_has_prefix (priv->popup_link_uri, "http://") ||
+                g_str_has_prefix (priv->popup_link_uri, "https://")) {
+                item = gtk_menu_item_new_with_mnemonic (_("_Copy Link Location"));
+                g_signal_connect (item, "activate",
+                                  G_CALLBACK (popup_copy_link), view);
+                gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
+            }
+            else {
+                item = gtk_menu_item_new_with_mnemonic (_("Open Link in New _Window"));
+                g_signal_connect (item, "activate",
+                                  G_CALLBACK (popup_open_link_new), view);
+                gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
+            }
 
             for (cur = priv->link_actions; cur != NULL; cur = cur->next) {
                 gboolean add;
@@ -2025,7 +2058,7 @@ document_callback (YelpDocument       *document,
     else if (signal == YELP_DOCUMENT_SIGNAL_CONTENTS) {
         YelpUriDocumentType doctype;
 	const gchar *contents;
-        gchar *mime_type, *page_id, *frag_id, *full_uri;
+        gchar *mime_type, *page_id, *frag_id, *full_uri, *search_terms;
         page_id = yelp_uri_get_page_id (priv->uri);
         debug_print (DB_ARG, "    document.uri.page_id=\"%s\"\n", page_id);
         mime_type = yelp_document_get_mime_type (document, page_id);
@@ -2079,17 +2112,20 @@ document_callback (YelpDocument       *document,
         g_signal_handler_unblock (view, priv->navigation_requested);
         g_object_set (view, "state", YELP_VIEW_STATE_LOADED, NULL);
 
+        search_terms = yelp_uri_get_query (priv->uri, "terms");
+
         /* If we need to set the GtkAdjustment or trigger the page title
          * from what WebKit thinks it is (see comment below), we need to
          * let the main loop run through.
          */
-        if (priv->vadjust > 0 || priv->hadjust > 0 || priv->page_title == NULL)
+        if (priv->vadjust > 0 || priv->hadjust > 0 ||
+            priv->page_title == NULL || search_terms != NULL)
             while (g_main_context_pending (NULL)) {
                 WebKitLoadStatus status;
                 status = webkit_web_view_get_load_status (WEBKIT_WEB_VIEW (view));
                 g_main_context_iteration (NULL, FALSE);
                 /* Sometimes some runaway JavaScript causes there to always
-                 * be pending sources. Break out of the document is loaded.
+                 * be pending sources. Break out if the document is loaded.
                  */
                 if (status == WEBKIT_LOAD_FINISHED ||
                     status == WEBKIT_LOAD_FAILED)
@@ -2125,6 +2161,36 @@ document_callback (YelpDocument       *document,
             g_signal_emit_by_name (view, "notify::page-title", spec);
         }
 
+        if (search_terms) {
+            WebKitDOMDocument *doc;
+            WebKitDOMElement *body, *div, *link;
+            doc = webkit_web_view_get_dom_document (WEBKIT_WEB_VIEW (view));
+            body = webkit_dom_document_query_selector (doc, "div.body", NULL);
+            if (body) {
+                gchar *tmp, *uri, *txt;
+                div = webkit_dom_document_create_element (doc, "div", NULL);
+                webkit_dom_element_set_attribute (div, "class", "fullsearch", NULL);
+                link = webkit_dom_document_create_element (doc, "a", NULL);
+                tmp = g_uri_escape_string (search_terms, NULL, FALSE);
+                uri = g_strconcat ("xref:search=", tmp, NULL);
+                webkit_dom_element_set_attribute (link, "href", uri, NULL);
+                g_free (tmp);
+                g_free (uri);
+                txt = g_strdup_printf (_("See all search results for “%s”"),
+                                       search_terms);
+                webkit_dom_node_set_text_content (WEBKIT_DOM_NODE (link), txt, NULL);
+                g_free (txt);
+                webkit_dom_node_append_child (WEBKIT_DOM_NODE (div),
+                                              WEBKIT_DOM_NODE (link),
+                                              NULL);
+                webkit_dom_node_insert_before (WEBKIT_DOM_NODE (body),
+                                               WEBKIT_DOM_NODE (div),
+                                               webkit_dom_node_get_first_child (WEBKIT_DOM_NODE (body)),
+                                               NULL);
+            }
+            g_free (search_terms);
+        }
+
         g_free (frag_id);
         g_free (page_id);
         g_free (mime_type);
